/* tclmoreDecls.h --
   
   Part of: tclmore
   Contents: declarations
   Date: Mon Aug 25, 2003
   
   Abstract
   
	This header files declares all the function prototypes and the
	corresponding preprocessor macros, to be used by stub enabled
	libraries.
   
   Copyright (c) 2002, 2003, 2004 Marco Maggi
   
   The author  hereby grant permission to use,  copy, modify, distribute,
   and  license this  software  and its  documentation  for any  purpose,
   provided that  existing copyright notices  are retained in  all copies
   and that  this notice  is included verbatim  in any  distributions. No
   written agreement, license, or royalty  fee is required for any of the
   authorized uses.  Modifications to this software may be copyrighted by
   their authors and need not  follow the licensing terms described here,
   provided that the new terms are clearly indicated on the first page of
   each file where they apply.
   
   IN NO  EVENT SHALL THE AUTHOR  OR DISTRIBUTORS BE LIABLE  TO ANY PARTY
   FOR  DIRECT, INDIRECT, SPECIAL,  INCIDENTAL, OR  CONSEQUENTIAL DAMAGES
   ARISING OUT  OF THE  USE OF THIS  SOFTWARE, ITS DOCUMENTATION,  OR ANY
   DERIVATIVES  THEREOF, EVEN  IF THE  AUTHOR  HAVE BEEN  ADVISED OF  THE
   POSSIBILITY OF SUCH DAMAGE.
   
   THE  AUTHOR  AND DISTRIBUTORS  SPECIFICALLY  DISCLAIM ANY  WARRANTIES,
   INCLUDING,   BUT   NOT  LIMITED   TO,   THE   IMPLIED  WARRANTIES   OF
   MERCHANTABILITY,    FITNESS   FOR    A    PARTICULAR   PURPOSE,    AND
   NON-INFRINGEMENT.  THIS  SOFTWARE IS PROVIDED  ON AN "AS  IS" BASIS,
   AND  THE  AUTHOR  AND  DISTRIBUTORS  HAVE  NO  OBLIGATION  TO  PROVIDE
   MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
   
*/

#ifndef __TCLMOREDECLS_H
#define __TCLMOREDECLS_H 1


/** ------------------------------------------------------------
 ** Preprocessor symbols for language declarations.
 ** ----------------------------------------------------------*/

#undef TCLMORE_BEGIN_C_DECL
#undef TCLMORE_END_C_DECL
#ifdef __cplusplus
#  define TCLMORE_BEGIN_C_DECL		extern "C" {
#  define TCLMORE_END_C_DECL		}
#else
#  define TCLMORE_BEGIN_C_DECL		/* empty */
#  define TCLMORE_END_C_DECL		/* empty */
#endif

/*
  Debug messages macro. Usage example:

	tclmore_debug(("variable foo \"%d\", foo));

  TCLMORE_DEBUG_MESSAGES  should  be declared  on  the  command line  of
  "make"; TCLMORE_DEBUGGING should be defined true in the target file of
  the debug session.

  Note. Including the __FILE__ symbol in the output message makes it too
  long (the directory part of the file name is included also).

  Note. For some reason when using the tcltest package, stdout is better
  than stderr.
*/

#ifndef TCLMORE_DEBUG_CHANNEL
#   define TCLMORE_DEBUG_CHANNEL		stdout
#endif

#undef More_Debug
#ifdef TCLMORE_DEBUG_MESSAGES
#   define More_Debug(...)	(void)(TCLMORE_DEBUGGING? \
		(fprintf(TCLMORE_DEBUG_CHANNEL, "%-25s*** ", __func__), \
		 fprintf(TCLMORE_DEBUG_CHANNEL, __VA_ARGS__), \
		 fprintf(TCLMORE_DEBUG_CHANNEL, "\n"), \
		 fflush(TCLMORE_DEBUG_CHANNEL)) : 0)
#else
#   define More_Debug(...)
#endif

#undef  More_Question
#ifdef TCLMORE_DEBUG_MESSAGES
#   define More_Question(expr)		((expr)? "yes" : "no")
#else
#   define More_Question(expr)
#endif


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#include <tcl.h>
#include <string.h>
#include <stdarg.h>
#include <stddef.h>

TCLMORE_BEGIN_C_DECL


/** ------------------------------------------------------------
 ** Symbols and type declarations.
 ** ----------------------------------------------------------*/

typedef Tcl_Obj *CONST More_ObjStack[];

#define __TCL_COMMAND_ARGS__	\
   ClientData D, Tcl_Interp *interp, int objc, More_ObjStack objv


#ifndef offsetof
#   define offsetof(type, member)	\
	((size_t) ((char *) &(((type *) 0)->member)))
#endif

/*
  Note: just  to be sure  do not remove  the cast to "ptrdiff_t"  in the
  following macro.
*/
#undef  More_AddOffset
#define More_AddOffset(base, offset)	((char *) base + (ptrdiff_t) offset)

#ifndef HAVE_PTRDIFF_T
typedef int		ptrdiff_t;
#endif

#undef  More_Byte
#define More_Byte		unsigned char
#undef  More_BytePtr
#define More_BytePtr		More_Byte *


#define More_AllocStruct(ptr, Type)		\
		ptr = (Type *) ckalloc(sizeof(Type));	\
		memset(ptr, '\0', sizeof(Type));


/** ------------------------------------------------------------
 ** String type declaration.
 ** ----------------------------------------------------------*/

/*
  This type exists mainly to be used by command argument extractors. The
  only reason not to use More_Block is the "char *" pointer.
*/

#define MORE_STRING_NULL_VALUE	{ 0, NULL }
typedef struct More_String {
  int		len;
  char *	ptr;
} More_String;


/** ------------------------------------------------------------
 ** Memory block macros.
 ** ----------------------------------------------------------*/

#define MORE_BLOCK_NULL_VALUE	{ 0, NULL }

typedef struct More_Block {
  int		len;
  More_BytePtr	ptr;
} More_Block;

#define More_BlockAlloc(block, size)		\
		((block).ptr = ckalloc((unsigned)(size)),	\
		 (block).len = (size))

#define More_BlockRealloc(block, newSize)	\
		((block).ptr = ckrealloc((block).ptr, (unsigned)(newSize)), \
		 (block).len = (newSize))

#define More_BlockFree(block)			\
		(ckfree((char *)(block).ptr), More_BlockReset(block))


#define More_BlockIsNull(block)		(! (block).ptr)

#define More_BlockReset(block)		((block).ptr = NULL, (block).len = 0)


#define More_BlockFromByteArray(block, objPtr)	\
		block.ptr = Tcl_GetByteArrayFromObj(objPtr, &block.len)

#define More_ByteArrayFromBlock(objPtr, block)	\
		objPtr = Tcl_NewByteArrayObj(block.ptr, block.len)



/** ------------------------------------------------------------
 ** Objects macros.
 ** ----------------------------------------------------------*/

typedef struct More_Objects {
  int		objc;
  Tcl_Obj **	objv;
} More_Objects;

#define MORE_OBJECTS_NULL_VALUE		{ 0, NULL }

#define More_ObjectInit(objects, _objc, _objv)	\
		(objects).objc = (_objc); (objects).objv = (_objv);

#define More_ObjectCount(objects)	(objects).objc
#define More_ObjectVector(objects)	(objects).objv


/** ------------------------------------------------------------
 ** Error handling type declarations.
 ** ----------------------------------------------------------*/

struct More_ErrorTag {
  CONST char *	errorCode;
  Tcl_Obj *	errorInfo;
  union More_ErrorData {
    ClientData	data;
    int		integer;
  } data;  
};
typedef struct More_ErrorTag *		More_Error;

#define More_ErrorSetData(error, _data)	((error)->data.data)=(_data)
#define More_ErrorGetData(error)	((error)->data.data)
#define More_ErrorSetInt(error, _integer) ((error)->data.integer)=(_integer)
#define More_ErrorGetInt(error)		((error)->data.integer)

#ifndef USE_TCLMORE_STUB

EXTERN More_Error More_ErrorNew		_ANSI_ARGS_((void));
EXTERN void More_ErrorForget		_ANSI_ARGS_((More_Error e));
EXTERN int  More_ErrorResult		_ANSI_ARGS_((Tcl_Interp *interp,
						     More_Error e));
EXTERN void More_ErrorLogic		_ANSI_ARGS_((More_Error e,
						     Tcl_Obj *info));
EXTERN void More_ErrorRuntime		_ANSI_ARGS_((More_Error e,
						     Tcl_Obj *info));
EXTERN void More_ErrorLogicStr		_ANSI_ARGS_((More_Error e,
						     CONST char *errorInfo));
EXTERN void More_ErrorRuntimeStr	_ANSI_ARGS_((More_Error e,
						     CONST char *errorInfo));
EXTERN void More_ErrorPrepend		_ANSI_ARGS_((More_Error e,
						     Tcl_Obj *errorInfo));
EXTERN void More_ErrorPrependStr	_ANSI_ARGS_((More_Error e,
						     CONST char *errorInfo));

EXTERN int More_ErrorIsLogic		_ANSI_ARGS_((More_Error e));
EXTERN int More_ErrorIsRuntime		_ANSI_ARGS_((More_Error e));

EXTERN void More_ErrorSetLogic		_ANSI_ARGS_((More_Error error));
EXTERN void More_ErrorSetRuntime	_ANSI_ARGS_((More_Error error));


EXTERN More_Error More_ErrorNoMemory	_ANSI_ARGS_((void));
EXTERN More_Error More_ErrorErrno	_ANSI_ARGS_((void));
EXTERN int More_ErrorCodeAndForget	_ANSI_ARGS_((More_Error error));

EXTERN int More_LogicError		_ANSI_ARGS_((Tcl_Interp *interp));
EXTERN int More_LogicErrorStr		_ANSI_ARGS_((Tcl_Interp *interp,
						     CONST char *errorInfo));
EXTERN int More_RuntimeError		_ANSI_ARGS_((Tcl_Interp *interp));
EXTERN int More_RuntimeErrorStr		_ANSI_ARGS_((Tcl_Interp *interp,
						     CONST char *errorInfo));

EXTERN int More_WrongNumArgs		_ANSI_ARGS_((Tcl_Interp *interp,
						     int objcount,
						     More_ObjStack objv,
						     CONST char *message));
EXTERN int More_OptionRequiresArg	_ANSI_ARGS_((Tcl_Interp *interp,
						     CONST char *option));

#else /* defined USE_TCLMORE_STUB */

#define More_ErrorNew			(moreStubErrorPtr->more_ErrorNew)
#define More_ErrorForget		(moreStubErrorPtr->more_ErrorForget)
#define More_ErrorResult		(moreStubErrorPtr->more_ErrorResult)

#define More_ErrorLogic			(moreStubErrorPtr->more_ErrorLogic)
#define More_ErrorRuntime		(moreStubErrorPtr->more_ErrorRuntime)
#define More_ErrorLogicStr		(moreStubErrorPtr->more_ErrorLogicStr)
#define More_ErrorRuntimeStr		(moreStubErrorPtr->more_ErrorRuntimeStr)
#define More_ErrorPrepend		(moreStubErrorPtr->more_ErrorPrepend)
#define More_ErrorPrependStr		(moreStubErrorPtr->more_ErrorPrependStr)
#define More_ErrorIsLogic		(moreStubErrorPtr->more_ErrorIsLogic)
#define More_ErrorIsRuntime		(moreStubErrorPtr->more_ErrorIsRuntime)
#define More_ErrorSetLogic		(moreStubErrorPtr->more_ErrorSetLogic)
#define More_ErrorSetRuntime		(moreStubErrorPtr->more_ErrorSetRuntime)

#define More_LogicError			(moreStubErrorPtr->more_LogicError)
#define More_LogicErrorStr		(moreStubErrorPtr->more_LogicErrorStr)
#define More_RuntimeError		(moreStubErrorPtr->more_RuntimeError)
#define More_RuntimeErrorStr		(moreStubErrorPtr->more_RuntimeErrorStr)

#define More_ErrorNoMemory		(moreStubErrorPtr->more_ErrorNoMemory)
#define More_ErrorErrno			(moreStubErrorPtr->more_ErrorErrno)
#define More_WrongNumArgs		(moreStubErrorPtr->more_WrongNumArgs)
#define More_OptionRequiresArg		(moreStubErrorPtr->more_OptionRequiresArg)

#endif /* defined USE_TCLMORE_STUB */

typedef struct More_StubTableError {

  More_Error	(*more_ErrorNew)	_ANSI_ARGS_((void));
  void		(*more_ErrorForget)	_ANSI_ARGS_((More_Error error));
  int		(*more_ErrorResult)	_ANSI_ARGS_((Tcl_Interp *interp,
						     More_Error error));

  void		(*more_ErrorLogic)	_ANSI_ARGS_((More_Error error,
						     Tcl_Obj *errorInfo));
  void		(*more_ErrorRuntime)	_ANSI_ARGS_((More_Error error,
						     Tcl_Obj *errorInfo));
  void		(*more_ErrorLogicStr)	_ANSI_ARGS_((More_Error error,
						     CONST char *errorInfo));
  void		(*more_ErrorRuntimeStr)	_ANSI_ARGS_((More_Error error,
						     CONST char *errorInfo));

  void		(*more_ErrorPrepend)	_ANSI_ARGS_((More_Error e,
						     Tcl_Obj *info));
  void		(*more_ErrorPrependStr)	_ANSI_ARGS_((More_Error e,
						     CONST char *info));

  int		(*more_ErrorIsLogic)	_ANSI_ARGS_((More_Error e));
  int		(*more_ErrorIsRuntime)	_ANSI_ARGS_((More_Error e));

  void		(*more_ErrorSetLogic)	_ANSI_ARGS_((More_Error error));
  void		(*more_ErrorSetRuntime)	_ANSI_ARGS_((More_Error error));

  More_Error	(*more_ErrorNoMemory)	_ANSI_ARGS_((void));
  More_Error	(*more_ErrorErrno)	_ANSI_ARGS_((void));
  int		(*more_ErrorCodeAndForget) _ANSI_ARGS_((More_Error error));

  int		(*more_LogicError)	_ANSI_ARGS_((Tcl_Interp *interp));
  int		(*more_LogicErrorStr)	_ANSI_ARGS_((Tcl_Interp *interp,
						     CONST char *errorInfo));
  int		(*more_RuntimeError)	_ANSI_ARGS_((Tcl_Interp *interp));
  int		(*more_RuntimeErrorStr)	_ANSI_ARGS_((Tcl_Interp *interp,
						     CONST char *errorInfo));

  int		(*more_WrongNumArgs)	_ANSI_ARGS_((Tcl_Interp *interp,
						     int objc,
						     Tcl_Obj *CONST objv[],
						     CONST char *message));

  int		(*more_OptionRequiresArg) _ANSI_ARGS_((Tcl_Interp *interp,
						       CONST char *option));
} More_StubTableError;

#ifdef USE_TCLMORE_STUB

EXTERN CONST More_StubTableError *CONST		moreStubErrorPtr;

#endif


/** ------------------------------------------------------------
 ** Command interface type declarations.
 ** ----------------------------------------------------------*/

struct More_CommandFrame {
  ClientData		stateStructPtr;
  ClientData		D;
  Tcl_Interp *		interp;
  int			objc;
  Tcl_Obj *CONST *	objv;
};
typedef struct More_CommandFrame * 	More_CommandFrame;
typedef int More_CmdFunc	_ANSI_ARGS_((More_CommandFrame frame));

struct More_ExtractorFrame {
  ClientData		fieldPtr;
  Tcl_Obj *		srcObj;
  More_CommandFrame	commandFrame;
};
typedef struct More_ExtractorFrame *	More_ExtractorFrame;
typedef int More_Extractor	_ANSI_ARGS_((More_ExtractorFrame frame));

typedef struct More_CommandOption {
  CONST char *		name;
  int			hasValue;
  More_Extractor *	extractor;
  ptrdiff_t		offsetInStateStruct;
} More_CommandOption;

typedef struct More_CommandArgument {
  More_Extractor *	extractor;
  ptrdiff_t		offsetInStateStruct;
} More_CommandArgument;

typedef struct More_CommandTreeLeaf {
  int				stackOffsetOfFirstArgument;
  More_CmdFunc *		function;
  unsigned			stateSize;
  CONST ClientData		stateDefaults;
  unsigned			argumentNumber;
  CONST More_CommandArgument *	argumentTable;
  CONST More_CommandOption *	optionTable;
  char *			synopsis;
} More_CommandTreeLeaf;

struct More_CommandTreeNode;
typedef CONST struct More_CommandTreeNode	More_CommandTreeNode;

struct More_CommandTreeNode {
  CONST char *			name;
  int				isSafeCommand;
  More_CommandTreeNode *	subcommands;
  More_CommandTreeLeaf *	definition;
};

#define TCLMORE_SAFE_COMMAND		1
#define TCLMORE_UNSAFE_COMMAND		0

#ifndef USE_TCLMORE_STUB

EXTERN More_Error More_CreateCommands	_ANSI_ARGS_((Tcl_Interp *interp,
						     CONST char *namespName,
						     More_CommandTreeNode *table));

#else /* defined USE_TCLMORE_STUB */

#define More_CreateCommands		(moreStubPtr->more_CreateCommands)

#endif /* end defined USE_TCLMORE_STUB */


/** ------------------------------------------------------------
 ** Identifiers table type declarations.
 ** ----------------------------------------------------------*/

typedef void	(*More_IdDestructor)	_ANSI_ARGS_((ClientData D));

typedef struct More_IdTable {
  Tcl_HashTable		table;
  unsigned		counter;
  CONST char *		tmplPtr;
  unsigned		tmplLen;
  More_IdDestructor	destructor;
} More_IdTable;


#ifndef USE_TCLMORE_STUB

EXTERN void	More_InitIdTable	_ANSI_ARGS_((More_IdTable *table,
						     CONST char *tmplPtr,
						     More_IdDestructor func));

EXTERN void	More_DeleteIdTable	_ANSI_ARGS_((More_IdTable *table));

EXTERN Tcl_Obj *More_AttachId		_ANSI_ARGS_((More_IdTable *table,
						     ClientData D));

EXTERN void	More_DetachId		_ANSI_ARGS_((More_IdTable *table,
						     Tcl_Obj *idPtr));

EXTERN ClientData More_GetDataFromId	_ANSI_ARGS_((More_IdTable *table,
						     Tcl_Obj *idPtr));

#else /* USE_TCLMORE_STUB */

#define More_InitIdTable	(moreStubIdTablePtr->more_InitIdTable)
#define More_DeleteIdTable	(moreStubIdTablePtr->more_DeleteIdTable)
#define More_AttachId		(moreStubIdTablePtr->more_AttachId)
#define More_DetachId		(moreStubIdTablePtr->more_DetachId)
#define	More_GetDataFromId	(moreStubIdTablePtr->more_GetDataFromId)

#endif	/* defined USE_TCLMORE_STUB */

typedef struct More_StubTableIdTable {

  void		(*more_InitIdTable) _ANSI_ARGS_((More_IdTable *table,
						 CONST char *tmplPtr,
						 More_IdDestructor destructor));

  void		(*more_DeleteIdTable) _ANSI_ARGS_((More_IdTable *table));

  Tcl_Obj *	(*more_AttachId) _ANSI_ARGS_((More_IdTable *table,
					      ClientData D));

  void		(*more_DetachId) _ANSI_ARGS_((More_IdTable *table,
					      Tcl_Obj *idPtr));

  ClientData	(*more_GetDataFromId) _ANSI_ARGS_((More_IdTable *table,
						   Tcl_Obj *idPtr));

} More_StubTableIdTable;

#ifdef USE_TCLMORE_STUB

EXTERN CONST More_StubTableIdTable *CONST	moreStubIdTablePtr;

#endif


/** ------------------------------------------------------------
 ** Object Extractor functions.
 ** ----------------------------------------------------------*/

#define TCLMORE_READ_OR_WRITE		1

#ifndef USE_TCLMORE_STUB

/* "unsigned" value. */

EXTERN Tcl_Obj * More_NewUnsignedObj	_ANSI_ARGS_((unsigned value));

EXTERN int More_GetUnsignedFromObj	_ANSI_ARGS_((Tcl_Interp *interp,
						     Tcl_Obj *objPtr,
						     unsigned *valVar));

EXTERN int \
       More_GetUnsignedInRangeFromObj	_ANSI_ARGS_((Tcl_Interp *interp,
						     Tcl_Obj *objPtr,
						     unsigned min, unsigned max,
						     unsigned * valVar));

/* "int" value. */

EXTERN int More_GetIntInRangeFromObj	_ANSI_ARGS_((Tcl_Interp *interp,
						     Tcl_Obj *objPtr,
						     int min, int max,
						     int * valVar));

/* "Tcl_WideInt" value. */

EXTERN int \
       More_GetWideIntInRangeFromObj	 _ANSI_ARGS_((Tcl_Interp *interp,
						      Tcl_Obj *objPtr,
						      Tcl_WideInt min,
						      Tcl_WideInt max,
						      Tcl_WideInt *valVar));

/* "size_t" value. */

EXTERN Tcl_Obj * More_NewSizeTObj	_ANSI_ARGS_((size_t value));

EXTERN int More_GetSizeTFromObj		_ANSI_ARGS_((Tcl_Interp *interp,
						     Tcl_Obj *objPtr,
						     size_t *valVar));

EXTERN int More_GetSizeTInRangeFromObj	_ANSI_ARGS_((Tcl_Interp *interp,
						     Tcl_Obj *objPtr,
						     size_t min,
						     size_t max,
						     size_t *valVar));


/* "float" value. */

EXTERN Tcl_Obj * More_NewFloatObj	_ANSI_ARGS_((float value));

EXTERN int More_GetFloatFromObj		_ANSI_ARGS_((Tcl_Interp *interp,
						     Tcl_Obj *objPtr,
						     float *valVar));

EXTERN int More_GetFloatInRangeFromObj	_ANSI_ARGS_((Tcl_Interp *interp,
						     Tcl_Obj *objPtr,
						     float min,
						     float max,
						     float *valVar));

#else

#define More_NewUnsignedObj		\
		(moreStubObjectExtractorPtr->more_NewUnsignedObj)
#define More_GetUnsignedFromObj		\
		(moreStubObjectExtractorPtr->more_GetUnsignedFromObj)
#define More_GetUnsignedInRangeFromObj	\
		(moreStubObjectExtractorPtr->more_GetUnsignedInRangeFromObj)
#define More_GetIntInRangeFromObj	\
		(moreStubObjectExtractorPtr->more_GetIntInRangeFromObj)
#define More_GetWideIntInRangeFromObj	\
		(moreStubObjectExtractorPtr->more_GetWideIntInRangeFromObj)
#define More_NewSizeTObj		\
		(moreStubObjectExtractorPtr->more_NewSizeTObj)
#define More_GetSizeTFromObj		\
		(moreStubObjectExtractorPtr->more_GetSizeTFromObj)
#define More_GetSizeTInRangeFromObj	\
		(moreStubObjectExtractorPtr->more_GetSizeTInRangeFromObj)
#define More_NewFloatObj		\
		(moreStubObjectExtractorPtr->more_NewFloatObj)
#define More_GetFloatFromObj		\
		(moreStubObjectExtractorPtr->more_GetFloatFromObj)
#define More_GetFloatInRangeFromObj	\
		(moreStubObjectExtractorPtr->more_GetFloatInRangeFromObj)

#endif	/* defined USE_TCLMORE_STUB */

typedef struct More_StubTableObjectExtractor {

  Tcl_Obj *(*more_NewUnsignedObj)	_ANSI_ARGS_((unsigned value));
  int	(*more_GetUnsignedFromObj)		\
		_ANSI_ARGS_((Tcl_Interp *interp, Tcl_Obj *objPtr, \
			     unsigned *valVar));
  int	(*more_GetUnsignedInRangeFromObj)	\
		_ANSI_ARGS_((Tcl_Interp *interp, Tcl_Obj *objPtr, \
			     unsigned min, unsigned max, unsigned * valVar));
  int	(*more_GetIntInRangeFromObj)		\
		_ANSI_ARGS_((Tcl_Interp *interp, Tcl_Obj *objPtr, \
			     int min, int max, int * valVar));

  int	(*more_GetWideIntInRangeFromObj)	\
		_ANSI_ARGS_((Tcl_Interp *interp, Tcl_Obj *objPtr, \
			     Tcl_WideInt min, Tcl_WideInt max, Tcl_WideInt *valVar));

  Tcl_Obj *	(*more_NewSizeTObj)		_ANSI_ARGS_((size_t value));
  int	(*more_GetSizeTFromObj)			\
		_ANSI_ARGS_((Tcl_Interp *interp, Tcl_Obj *objPtr, \
			     size_t *valVar));
  int	(*more_GetSizeTInRangeFromObj)		\
		_ANSI_ARGS_((Tcl_Interp *interp, Tcl_Obj *objPtr, \
			     size_t min, size_t max, size_t *valVar));

  Tcl_Obj *	(*more_NewFloatObj)		_ANSI_ARGS_((float value));

  int	(*more_GetFloatFromObj)			\
		_ANSI_ARGS_((Tcl_Interp *interp, Tcl_Obj *objPtr, \
			     float *valVar));
  int	(*more_GetFloatInRangeFromObj)		\
		_ANSI_ARGS_((Tcl_Interp *interp, Tcl_Obj *objPtr, \
			     float min, float max, float *valVar));
} More_StubTableObjectExtractor;

#ifdef USE_TCLMORE_STUB

EXTERN CONST More_StubTableObjectExtractor *CONST moreStubObjectExtractorPtr;

#endif


/** ------------------------------------------------------------
 ** Argument extractor functions.
 ** ----------------------------------------------------------*/

#ifndef USE_TCLMORE_STUB

EXTERN More_Extractor	More_GetObjFromArg;
EXTERN More_Extractor	More_GetStringFromArg;
EXTERN More_Extractor	More_GetAStringFromArg;
EXTERN More_Extractor	More_GetBlockFromArg;
EXTERN More_Extractor	More_GetABlockFromArg;

EXTERN More_Extractor	More_GetIntFromArg;
EXTERN More_Extractor	More_GetWideIntFromArg;
EXTERN More_Extractor	More_GetUnsignedFromArg;
EXTERN More_Extractor	More_GetFloatFromArg;
EXTERN More_Extractor	More_GetDoubleFromArg;
EXTERN More_Extractor	More_GetSizeTFromArg;

#else /* defined USE_TCLMORE_STUB */

#define More_GetObjFromArg		\
		(moreStubArgumentExtractorPtr->more_GetObjFromArg)
#define More_GetStringFromArg		\
		(moreStubArgumentExtractorPtr->more_GetStringFromArg)
#define More_GetAStringFromArg		\
		(moreStubArgumentExtractorPtr->more_GetAStringFromArg)
#define More_GetBlockFromArg		\
		(moreStubArgumentExtractorPtr->more_GetBlockFromArg)
#define More_GetABlockFromArg		\
		(moreStubArgumentExtractorPtr->more_GetABlockFromArg)
#define More_GetIntFromArg		\
		(moreStubArgumentExtractorPtr->more_GetIntFromArg)
#define More_GetWideIntFromArg		\
		(moreStubArgumentExtractorPtr->more_GetWideIntFromArg)
#define More_GetUnsignedFromArg		\
		(moreStubArgumentExtractorPtr->more_GetUnsignedFromArg)
#define More_GetFloatFromArg		\
		(moreStubArgumentExtractorPtr->more_GetFloatFromArg)
#define More_GetDoubleFromArg		\
		(moreStubArgumentExtractorPtr->more_GetDoubleFromArg)
#define More_GetSizeTFromArg		\
		(moreStubArgumentExtractorPtr->more_GetSizeTFromArg)

#endif	/* end defined USE_TCLMORE_STUB */

typedef struct More_StubTableArgumentExtractor {
  More_Extractor	(*more_GetObjFromArg);
  More_Extractor	(*more_GetStringFromArg);
  More_Extractor	(*more_GetAStringFromArg);
  More_Extractor	(*more_GetBlockFromArg);
  More_Extractor	(*more_GetABlockFromArg);
  More_Extractor	(*more_GetIntFromArg);
  More_Extractor	(*more_GetWideIntFromArg);
  More_Extractor	(*more_GetUnsignedFromArg);
  More_Extractor	(*more_GetFloatFromArg);
  More_Extractor	(*more_GetDoubleFromArg);
  More_Extractor	(*more_GetSizeTFromArg);
} More_StubTableArgumentExtractor;

#ifdef USE_TCLMORE_STUB

EXTERN CONST More_StubTableArgumentExtractor *CONST moreStubArgumentExtractorPtr;

#endif


/** ------------------------------------------------------------
 ** Callback type declaration.
 ** ----------------------------------------------------------*/

typedef void	More_CallbackFunction	_ANSI_ARGS_((ClientData D));

typedef struct More_Callback {
  ClientData			data;
  More_CallbackFunction	*	function;
} More_Callback;

#define More_CallbackInit(callback, _function, _data)	\
		(callback).function = (_function); \
		(callback).data     = (ClientData)(_data)
#define	More_CallbackPresent(callback)	((callback).function)
#define More_CallbackReset(callback)	((callback).function = NULL)
#define More_CallbackInvoke(callback)	((callback).function((callback).data))


/** ------------------------------------------------------------
 ** Variable stuff.
 ** ----------------------------------------------------------*/

typedef struct More_Variable {
  Tcl_Interp *	interp;
  Tcl_Obj *	name;
} More_Variable[1];

#ifndef USE_TCLMORE_STUB

EXTERN void	More_VariableInit	_ANSI_ARGS_((More_Variable variable,
						     Tcl_Interp *interp,
						     Tcl_Obj *variableName));
EXTERN void	More_VariableCopy	_ANSI_ARGS_((More_Variable dest,
						     More_Variable source));
EXTERN void	More_VariableFinal	_ANSI_ARGS_((More_Variable variable));
EXTERN int	More_VariableTest	_ANSI_ARGS_((More_Variable variable));
EXTERN int	More_VariableExists	_ANSI_ARGS_((More_Variable variable));

EXTERN void	More_VariableSet	_ANSI_ARGS_((More_Variable variable,
						     Tcl_Obj *objPtr));
EXTERN void	More_VariableSetBlock	_ANSI_ARGS_((More_Variable variable,
						     More_Block block));
EXTERN void	More_VariableAppend	_ANSI_ARGS_((More_Variable variable,
						     Tcl_Obj *objPtr));
EXTERN void	More_VariableAppendBlock _ANSI_ARGS_((More_Variable variable,
						      More_Block block));
EXTERN void	More_VariableClear	_ANSI_ARGS_((More_Variable variable));
EXTERN Tcl_Obj *More_VariableGet	_ANSI_ARGS_((More_Variable variable));
EXTERN More_Block More_VariableGetBlock _ANSI_ARGS_((More_Variable variable));

EXTERN CONST char * More_VariableName	_ANSI_ARGS_((More_Variable variable));

#else /* defined USE_TCLMORE_STUB */

#define More_VariableInit	(moreStubVariablePtr->more_VariableInit)
#define More_VariableCopy	(moreStubVariablePtr->more_VariableCopy)
#define More_VariableFinal	(moreStubVariablePtr->more_VariableFinal)
#define More_VariableTest	(moreStubVariablePtr->more_VariableTest)
#define More_VariableExists	(moreStubVariablePtr->more_VariableExists)
#define More_VariableSet	(moreStubVariablePtr->more_VariableSet)
#define More_VariableSetBlock	(moreStubVariablePtr->more_VariableSetBlock)
#define More_VariableAppend	(moreStubVariablePtr->more_VariableAppend)
#define More_VariableAppendBlock (moreStubVariablePtr->more_VariableAppendBlock)
#define More_VariableClear	(moreStubVariablePtr->more_VariableClear)
#define More_VariableGet	(moreStubVariablePtr->more_VariableGet)
#define More_VariableGetBlock	(moreStubVariablePtr->more_VariableGetBlock)
#define More_VariableName	(moreStubVariablePtr->more_VariableName)

#endif	/* defined USE_TCLMORE_STUB */

typedef struct More_StubTableVariable {

  void	(*more_VariableInit)	_ANSI_ARGS_((More_Variable variable,
					     Tcl_Interp *interp,
					     Tcl_Obj *variableName));

  void	(*more_VariableCopy)	_ANSI_ARGS_((More_Variable dest,
					     More_Variable source));

  void	(*more_VariableFinal)	_ANSI_ARGS_((More_Variable variable));

  int	(*more_VariableTest)	_ANSI_ARGS_((More_Variable variable));

  int	(*more_VariableExists)	_ANSI_ARGS_((More_Variable variable));

  void	(*more_VariableSet)	_ANSI_ARGS_((More_Variable variable,
					     Tcl_Obj *objPtr));

  void	(*more_VariableSetBlock) _ANSI_ARGS_((More_Variable variable,
					      More_Block block));

  void	(*more_VariableAppend)	_ANSI_ARGS_((More_Variable variable,
					     Tcl_Obj *objPtr));

  void	(*more_VariableAppendBlock) _ANSI_ARGS_((More_Variable variable,
						 More_Block block));

  void	(*more_VariableClear)	_ANSI_ARGS_((More_Variable variable));

  Tcl_Obj * (*more_VariableGet)	_ANSI_ARGS_((More_Variable variable));

  More_Block (*more_VariableGetBlock)	_ANSI_ARGS_((More_Variable variable));

  CONST char * (*more_VariableName)	_ANSI_ARGS_((More_Variable variable));

} More_StubTableVariable;

#ifdef USE_TCLMORE_STUB

EXTERN CONST More_StubTableVariable *CONST	moreStubVariablePtr;

#endif


/** ------------------------------------------------------------
 ** Buffer functions.
 ** ----------------------------------------------------------*/

typedef ClientData		More_Buffer;

#ifndef USE_TCLMORE_STUB

EXTERN More_Buffer More_BufferAlloc	_ANSI_ARGS_((void));

EXTERN void	More_BufferFree		_ANSI_ARGS_((More_Buffer buffer));

EXTERN int	More_BufferRead		_ANSI_ARGS_((More_Buffer buffer,
						     More_Block  output));
EXTERN More_Block More_BufferReadAllBlock _ANSI_ARGS_((More_Buffer token));
EXTERN Tcl_Obj *  More_BufferReadAllObj   _ANSI_ARGS_((More_Buffer token));

EXTERN void	More_BufferWrite	_ANSI_ARGS_((More_Buffer buffer,
						     More_Block  input));

EXTERN int	More_BufferEmpty	_ANSI_ARGS_((More_Buffer buffer));
EXTERN int	More_BufferEof		_ANSI_ARGS_((More_Buffer buffer));
EXTERN int	More_BufferAlive	_ANSI_ARGS_((More_Buffer buffer));

EXTERN void More_BufferReaderCallback	_ANSI_ARGS_((More_Buffer buffer,
						     More_Callback callback));

EXTERN void More_BufferWriterCallback	_ANSI_ARGS_((More_Buffer buffer,
						     More_Callback callback));

EXTERN void	More_BufferDetachReader _ANSI_ARGS_((More_Buffer buffer));
EXTERN void	More_BufferDetachWriter _ANSI_ARGS_((More_Buffer buffer));

EXTERN void	More_BufferSetSize	_ANSI_ARGS_((More_Buffer buffer,
						     size_t bufferSize));
EXTERN size_t	More_BufferGetSize	_ANSI_ARGS_((More_Buffer buffer));

#else /* defined USE_TCLMORE_STUB */

#define More_BufferAlloc	(moreStubBufferPtr->more_BufferAlloc)
#define More_BufferFree		(moreStubBufferPtr->more_BufferFree)
#define More_BufferRead		(moreStubBufferPtr->more_BufferRead)
#define More_BufferReadAllBlock	(moreStubBufferPtr->more_BufferReadAllBlock)
#define More_BufferReadAllObj	(moreStubBufferPtr->more_BufferReadAllObj)
#define More_BufferWrite	(moreStubBufferPtr->more_BufferWrite)
#define More_BufferEmpty	(moreStubBufferPtr->more_BufferEmpty)
#define More_BufferEof		(moreStubBufferPtr->more_BufferEof)
#define More_BufferAlive	(moreStubBufferPtr->more_BufferAlive)
#define More_BufferReaderCallback (moreStubBufferPtr->more_BufferReaderCallback)
#define More_BufferWriterCallback (moreStubBufferPtr->more_BufferWriterCallback)
#define More_BufferDetachReader	(moreStubBufferPtr->more_BufferDetachReader)
#define More_BufferDetachWriter	(moreStubBufferPtr->more_BufferDetachWriter)
#define More_BufferSetSize	(moreStubBufferPtr->more_BufferSetSize)
#define More_BufferGetSize	(moreStubBufferPtr->more_BufferGetSize)

#endif /* end defined USE_TCLMORE_STUB */

typedef struct More_StubTableBuffer {
  More_Buffer 	(*more_BufferAlloc)	_ANSI_ARGS_((void));

  void		(*more_BufferFree)	_ANSI_ARGS_((More_Buffer buffer));

  int		(*more_BufferRead)	_ANSI_ARGS_((More_Buffer buffer,
						     More_Block  block));
  More_Block	(*more_BufferReadAllBlock) _ANSI_ARGS_((More_Buffer token));
  Tcl_Obj *	(*more_BufferReadAllObj)   _ANSI_ARGS_((More_Buffer token));

  void		(*more_BufferWrite)	_ANSI_ARGS_((More_Buffer buffer,
						     More_Block  block));

  int		(*more_BufferEmpty)	_ANSI_ARGS_((More_Buffer buffer));
  int		(*more_BufferEof)	_ANSI_ARGS_((More_Buffer buffer));
  int		(*more_BufferAlive)	_ANSI_ARGS_((More_Buffer buffer));


  void	(*more_BufferReaderCallback)	_ANSI_ARGS_((More_Buffer buffer,
						     More_Callback callback));
  void	(*more_BufferWriterCallback)	_ANSI_ARGS_((More_Buffer buffer,
						     More_Callback callback));
  void	(*more_BufferDetachReader)	_ANSI_ARGS_((More_Buffer buffer));
  void	(*more_BufferDetachWriter)	_ANSI_ARGS_((More_Buffer buffer));

  void		(*more_BufferSetSize)	_ANSI_ARGS_((More_Buffer buffer,
						     size_t bufferSize));
  size_t	(*more_BufferGetSize)	_ANSI_ARGS_((More_Buffer buffer));
} More_StubTableBuffer;

#ifdef USE_TCLMORE_STUB

EXTERN CONST More_StubTableBuffer *CONST	moreStubBufferPtr;

#endif


/** ------------------------------------------------------------
 ** Channel interfaces.
 ** ----------------------------------------------------------*/

/* Useful only for argument extraction. */

typedef struct More_ChannelArg {
  Tcl_Channel	token;
  int		modeMask;
} More_ChannelArg;

#define MORE_CHANNEL_ARGUMENT_DEFAULT	{ NULL, 0 }


#ifndef USE_TCLMORE_STUB

EXTERN ClientData \
	More_CreateBufferVariable	_ANSI_ARGS_((More_Variable variable,
						     More_Buffer input,
						     More_Buffer output));

EXTERN void More_DeleteBufferVariable	_ANSI_ARGS_((More_Variable variable,
						     ClientData	D));

EXTERN Tcl_Channel \
	  More_OpenBufferChannel	_ANSI_ARGS_((More_Buffer input,
						     More_Buffer output));


EXTERN Tcl_Channel More_OpenVarChannel	_ANSI_ARGS_((More_Variable variable,
						     int modeMask));

EXTERN Tcl_Channel More_StackTeeChannel	_ANSI_ARGS_((Tcl_Interp *interp,
						     Tcl_Channel subChannel,
						     int modeMask));

EXTERN Tcl_Channel More_OpenPipeChannel	_ANSI_ARGS_((int modeMask,
						     Tcl_Channel *channelVar));

EXTERN int More_GetOpenModeFromObj	_ANSI_ARGS_((Tcl_Interp *interp,
						     Tcl_Obj *objPtr,
						     int flags,
						     int *modeMask));

EXTERN int \
   More_GetTransformOpenModeFromObj	_ANSI_ARGS_((Tcl_Interp *interp,
						     Tcl_Obj *objPtr,
						     int subModeMask,
						     int flags,
						     int *modeMaskVar));

EXTERN int More_GetChannelFromObj	_ANSI_ARGS_((Tcl_Interp *interp,
						     Tcl_Obj *objPtr,
						     Tcl_Channel *channelVar,
						     int *modeMaskVar));

#else

#define More_CreateBufferVariable	\
		(moreStubChannelInterfacePtr->more_CreateBufferVariable)
#define More_DeleteBufferVariable	\
		(moreStubChannelInterfacePtr->more_DeleteBufferVariable)
#define More_OpenBufferChannel		\
		(moreStubChannelInterfacePtr->more_OpenBufferChannel)
#define More_OpenVarChannel		\
		(moreStubChannelInterfacePtr->more_OpenVarChannel)
#define More_StackTeeChannel		\
		(moreStubChannelInterfacePtr->more_StackTeeChannel)
#define More_OpenPipeChannel		\
		(moreStubChannelInterfacePtr->more_OpenPipeChannel)
#define More_GetOpenModeFromObj		\
		(moreStubChannelInterfacePtr->more_GetOpenModeFromObj)
#define More_GetTransformOpenModeFromObj	\
		(moreStubChannelInterfacePtr->more_GetTransformOpenModeFromObj)
#define More_GetChannelFromObj		\
		(moreStubChannelInterfacePtr->more_GetChannelFromObj)

#endif	/* defined USE_TCLMORE_STUB */

typedef struct More_StubTableChannelInterface {

  ClientData \
	(*more_CreateBufferVariable)	_ANSI_ARGS_((More_Variable variable,
					     More_Buffer inputBuffer,
					     More_Buffer outputBuffer));

  void (*more_DeleteBufferVariable)	_ANSI_ARGS_((More_Variable variable,
						     ClientData	D));

  Tcl_Channel (*more_OpenBufferChannel)	_ANSI_ARGS_((More_Buffer inputBuffer,
					    More_Buffer outputBuffer));

  
  Tcl_Channel (*more_OpenVarChannel)	_ANSI_ARGS_((More_Variable variable,
						     int modeMask));

  Tcl_Channel (*more_StackTeeChannel)	_ANSI_ARGS_((Tcl_Interp *interp,
						     Tcl_Channel subChannel,
						     int modeMask));

  Tcl_Channel	(*more_OpenPipeChannel) _ANSI_ARGS_((int modeMask,
						     Tcl_Channel *channelVar));

  int		(*more_GetOpenModeFromObj) _ANSI_ARGS_((Tcl_Interp *interp,
							Tcl_Obj *objPtr,
							int flags,
							int *modeMask));

  int (*more_GetTransformOpenModeFromObj) _ANSI_ARGS_((Tcl_Interp *interp,
						       Tcl_Obj *objPtr,
						       int subModeMask,
						       int flags,
						       int *modeMaskVar));

  int	(*more_GetChannelFromObj)	_ANSI_ARGS_((Tcl_Interp *interp,
						     Tcl_Obj *objPtr,
						     Tcl_Channel *channelVar,
						     int *modeMaskVar));
  Tcl_Channel (*more_StackPutbackChannel) _ANSI_ARGS_((Tcl_Interp *interp,
						       Tcl_Channel subChannel,
						       int modeMask));
  int	(*more_PutbackData)		_ANSI_ARGS_((Tcl_Channel transform,
						     int input,
						     More_Block block));

} More_StubTableChannelInterface;

#ifdef USE_TCLMORE_STUB

EXTERN CONST More_StubTableChannelInterface *CONST moreStubChannelInterfacePtr;

#endif


/** ------------------------------------------------------------
 ** Channel driver facilities.
 ** ----------------------------------------------------------*/

typedef int More_ChannelDriverSetOptionProc	\
			_ANSI_ARGS_((ClientData D,
				     Tcl_Interp *interp,
				     CONST char *optionName,
				     CONST char *optionValue));

typedef int More_ChannelDriverGetOptionProc	\
			_ANSI_ARGS_((ClientData D,
				     Tcl_Interp *interp,
				     CONST char *optionName,
				     Tcl_DString *optionValue));

typedef struct More_ChannelDriverOption {
  CONST char *				name;
  More_ChannelDriverSetOptionProc *	setOptionProc;
  More_ChannelDriverGetOptionProc *	getOptionProc;
} More_ChannelDriverOption;

#ifndef USE_TCLMORE_STUB

EXTERN int More_ChannelDriverSetOption		\
			 _ANSI_ARGS_((CONST More_ChannelDriverOption *table,
				      Tcl_Channel channel,
				      ClientData D,
				      Tcl_Interp *interp,
				      CONST char *optionName,
				      CONST char *optionValue));

EXTERN int More_ChannelDriverGetOption		\
			_ANSI_ARGS_((CONST More_ChannelDriverOption *table,
				     Tcl_Channel channel,
				     ClientData D,
				     Tcl_Interp *interp,
				     CONST char *optionName,
				     Tcl_DString *optionValue));

#else /* define USE_TCLMORE_STUB */

#define More_ChannelDriverSetOption	\
		(moreStubChannelDriverPtr->more_ChannelDriverSetOption)
#define More_ChannelDriverGetOption	\
		(moreStubChannelDriverPtr->more_ChannelDriverGetOption)

#endif /* end defined USE_TCLMORE_STUB */

typedef struct More_StubTableChannelDriver {
  int		(*more_ChannelDriverSetOption)		\
			 _ANSI_ARGS_((CONST More_ChannelDriverOption *table,
				      Tcl_Channel channel,
				      ClientData D,
				      Tcl_Interp *interp,
				      CONST char *optionName,
				      CONST char *optionValue));

  int		(*more_ChannelDriverGetOption)		\
			_ANSI_ARGS_((CONST More_ChannelDriverOption *table,
				     Tcl_Channel channel,
				     ClientData D,
				     Tcl_Interp *interp,
				     CONST char *optionName,
				     Tcl_DString *optionValue));
} More_StubTableChannelDriver;

#ifdef USE_TCLMORE_STUB

EXTERN CONST More_StubTableChannelDriver *CONST moreStubChannelDriverPtr;

#endif


/** ------------------------------------------------------------
 ** Stream transformation interface.
 ** ----------------------------------------------------------*/

typedef ClientData			More_Stream;

typedef void	   More_StreamFinal	_ANSI_ARGS_((More_Stream token));
typedef More_Block More_StreamOutput	_ANSI_ARGS_((More_Stream token));
typedef void	   More_StreamRead	_ANSI_ARGS_((More_Stream token,
						     int number));
typedef More_Error More_StreamWrite	_ANSI_ARGS_((More_Stream token,
						     More_Block *blockPtr));
typedef More_Error More_StreamFlush	_ANSI_ARGS_((More_Stream token));
typedef More_Error More_StreamFinish	_ANSI_ARGS_((More_Stream token));

typedef struct More_StreamDriver {
  CONST char *		version;
  CONST char *		type;
  More_StreamFinal	*final;
  More_StreamOutput	*output;
  More_StreamRead	*read;
  More_StreamWrite	*write;
  More_StreamFlush	*flush;
  More_StreamFinish	*finish;

  CONST More_ChannelDriverOption *optionTable;
} More_StreamDriver;

typedef struct More_StreamIO {
  More_Stream	input;
  More_Stream	output;
} More_StreamIO;

#ifndef USE_TCLMORE_STUB

EXTERN Tcl_Channel More_MakeStreamTransform	\
		_ANSI_ARGS_((CONST More_StreamDriver *streamPtr,
			     More_Stream input, More_Stream output,
			     Tcl_Channel subChannel));

#else /* defined USE_TCLMORE_STUB */

#define More_MakeStreamTransform		\
		(moreStubStreamPtr->more_MakeStreamTransform)

#endif /* end defined USE_TCLMORE_STUB */

typedef struct More_StubTableStream {

  Tcl_Channel (*more_MakeStreamTransform)	\
		 _ANSI_ARGS_((CONST More_StreamDriver *streamPtr,
			      More_Stream input, More_Stream output,
			      Tcl_Channel subChannel));  

} More_StubTableStream;

#ifdef USE_TCLMORE_STUB

EXTERN CONST More_StubTableStream *CONST moreStubStreamPtr;

#endif


/** ------------------------------------------------------------
 ** Dynamic string functions.
 ** ----------------------------------------------------------*/

#ifndef USE_TCLMORE_STUB

EXTERN void	More_DStringPrintf	_ANSI_ARGS_((Tcl_DString *string,
						     CONST char *format,
						     ...));

EXTERN void	More_DStringAppendSizeT _ANSI_ARGS_((Tcl_DString *string,
						     size_t size));

#else /* defined USE_TCLMORE_STUB */

#define More_DStringPrintf	(moreStubDStringPtr->more_DStringPrintf)
#define More_DStringAppendSizeT	(moreStubDStringPtr->more_DStringAppendSizeT)

#endif /* end defined USE_TCLMORE_STUB */


typedef struct More_StubTableDString {
  void	(*more_DStringPrintf)		_ANSI_ARGS_((Tcl_DString *string,
						     CONST char *format,
						     ...));
  void	(*more_DStringAppendSizeT)	_ANSI_ARGS_((Tcl_DString *string,
						     size_t size));
} More_StubTableDString;

#ifdef USE_TCLMORE_STUB

EXTERN CONST More_StubTableDString *CONST moreStubDStringPtr;

#endif


/** ------------------------------------------------------------
 ** Miscellaneous functions.
 ** ----------------------------------------------------------*/

#ifndef USE_TCLMORE_STUB

EXTERN int More_EqualVarnames	 _ANSI_ARGS_((CONST char *name,
					      CONST char *var,
					      CONST char *key));

EXTERN char * More_MakeName	 _ANSI_ARGS_((Tcl_Interp *interp,
					      CONST char *template));

EXTERN char * More_DupAllocString _ANSI_ARGS_((CONST char *string));

EXTERN Tcl_Obj * More_ObjPrintf	_ANSI_ARGS_((CONST char *format, ...));

EXTERN char * More_Asprintf _ANSI_ARGS_((CONST char *format, ...));

EXTERN char * More_Bsprintf	_ANSI_ARGS_((CONST char *format,
					     va_list ap));

#else /* defined USE_TCLMORE_STUB */

#define More_EqualVarnames	(moreStubMiscPtr->more_EqualVarnames)
#define More_MakeName		(moreStubMiscPtr->more_MakeName)
#define More_DupAllocString	(moreStubMiscPtr->more_DupAllocString)
#define More_Asprintf		(moreStubMiscPtr->more_Asprintf)
#define More_Bsprintf		(moreStubMiscPtr->more_Bsprintf)
#define More_ObjPrintf		(moreStubMiscPtr->more_ObjPrintf)

#endif /* end defined USE_TCLMORE_STUB */

typedef struct More_StubTableMisc {
  int		(*more_EqualVarnames)	_ANSI_ARGS_((CONST char *name,
						     CONST char *var,
						     CONST char *key));

  char *	(*more_MakeName)	_ANSI_ARGS_((Tcl_Interp *interp,
						     CONST char *template));

  char *	(*more_DupAllocString)	_ANSI_ARGS_((CONST char *string));

  char *	(*more_Asprintf)	_ANSI_ARGS_((CONST char *format, ...));
  char *	(*more_Bsprintf)	_ANSI_ARGS_((CONST char *format,
						     va_list ap));
  Tcl_Obj *	(*more_ObjPrintf)	_ANSI_ARGS_((CONST char *format, ...));
} More_StubTableMisc;

#ifdef USE_TCLMORE_STUB

EXTERN CONST More_StubTableMisc *CONST moreStubMiscPtr;

#endif


/** ------------------------------------------------------------
 ** Delayed script functions.
 ** ----------------------------------------------------------*/

typedef ClientData		More_DScript;

#ifndef USE_TCLMORE_STUB

EXTERN More_DScript More_DScriptInit	_ANSI_ARGS_((Tcl_Interp *interp,
						     Tcl_Obj *body));

EXTERN More_DScript More_DScriptCopy	_ANSI_ARGS_((More_DScript token));
EXTERN void More_DScriptFinal		_ANSI_ARGS_((More_DScript token));
EXTERN void More_DScriptQueue		_ANSI_ARGS_((More_DScript token,
						     More_Objects args));
EXTERN void More_DScriptCallback	_ANSI_ARGS_((More_DScript token,
						     More_Callback callback));

#else /* defined USE_TCLMORE_STUB */

#define More_DScriptInit	(moreStubDelayedScriptPtr)->more_DScriptInit
#define More_DScriptCopy	(moreStubDelayedScriptPtr)->more_DScriptCopy
#define More_DScriptFinal	(moreStubDelayedScriptPtr)->more_DScriptFinal
#define More_DScriptQueue	(moreStubDelayedScriptPtr)->more_DScriptQueue
#define More_DScriptCallback	(moreStubDelayedScriptPtr)->more_DScriptCallback

#endif /* end defined USE_TCLMORE_STUB */

typedef struct More_StubTableDelayedScript {
  More_DScript (*more_DScriptInit)	_ANSI_ARGS_((Tcl_Interp *interp,
						     Tcl_Obj *body));
  More_DScript (*more_DScriptCopy)	_ANSI_ARGS_((More_DScript token));
  void (*more_DScriptFinal)		_ANSI_ARGS_((More_DScript token));
  void (*more_DScriptQueue)		_ANSI_ARGS_((More_DScript token,
						     More_Objects args));
  void (*more_DScriptCallback)		_ANSI_ARGS_((More_DScript token,
						     More_Callback callback));
} More_StubTableDelayedScript;

#ifdef USE_TCLMORE_STUB

CONST More_StubTableDelayedScript *CONST moreStubDelayedScriptPtr;

#endif


/** ------------------------------------------------------------
 ** Stub table.
 ** ----------------------------------------------------------*/

typedef struct More_StubTable {

  CONST More_StubTableError *			moreStubErrorPtr;
  CONST More_StubTableIdTable *			moreStubIdTablePtr;
  CONST More_StubTableVariable *		moreStubVariablePtr;
  CONST More_StubTableChannelInterface *	moreStubChannelInterfacePtr;
  CONST More_StubTableChannelDriver *		moreStubChannelDriverPtr;
  CONST More_StubTableObjectExtractor *		moreStubObjectExtractorPtr;
  CONST More_StubTableArgumentExtractor *	moreStubArgumentExtractorPtr;
  CONST More_StubTableBuffer *			moreStubBufferPtr;
  CONST More_StubTableDString *			moreStubDStringPtr;
  CONST More_StubTableStream *			moreStubStreamPtr;
  CONST More_StubTableMisc *			moreStubMiscPtr;
  CONST More_StubTableDelayedScript *		moreStubDelayedScriptPtr;

  More_Error	(*more_CreateCommands)	_ANSI_ARGS_((Tcl_Interp *interp,
						CONST char *namespName,
						More_CommandTreeNode *table));
} More_StubTable;

#ifdef USE_TCLMORE_STUB

EXTERN CONST More_StubTable *CONST		moreStubPtr;

EXTERN CONST char *	More_InitStub	_ANSI_ARGS_((Tcl_Interp *interp,
						     char *version,
						     int exact));

#endif /* defined USE_TCLMORE_STUB */

TCLMORE_END_C_DECL

#endif /* __TCLMOREDECLS_H */

/* end of file */
